pro calcENVIProd, nSite, fInputDEM, ALL=ALL, SLOPE=SLOPE, ASPECT=ASPECT, PROFCONV=PROFCONV, SHRELIEF=SHRELIEF, KERNEL=KERNEL
	;Utiliza la funcion envi_doit para calcular distintos productos a partir de un MDE
	;nSite : nombre del sitio a procesar
	;fInputDEM : nombre del archivo que contiene el MDE
	;ALL : genera todos los subproductos
	;SLOPE : genera SLOPE
	;ASPECT : genera ASPECT
	;PROFCONV : genera PROFILE CONVEXITY
	;SHRELIEF: genera SHADED RELIEF
	;KERNEL : permite establecer la dimension del kernel

	readParam, getenv('paramFile'), 'ENVI', 'KERNEL', i_kernel
	readParam, getenv('paramFile'), 'ENVI', 'SHADED_RELIEF_AZIMUTH', i_shaded_relief_azimuth
	readParam, getenv('paramFile'), 'ENVI', 'SHADED_RELIEF_ELEVATION', i_shaded_relief_elevation

	bptr = lonarr(1)
	fOut = strarr(1)
	bptr[0] = -1
	fOut[0] = 'nn'
	shrelief_azimuth = !values.f_infinity
	shrelief_elevation = !values.f_infinity

	if keyword_set(SLOPE) then begin
		bptr = [bptr, 0]
		fOut = [fOut, 'slope']
	endif

	if keyword_set(ASPECT) then begin
		bptr = [bptr, 1]
		fOut = [fOut, 'aspect']
	endif

	if keyword_set(PROFCONV) then begin
		bptr = [bptr, 3]
		fOut = [fOut, 'profile_convexity']
	endif

	if keyword_set(SHRELIEF) then begin
		bptr = [bptr, 2]
		fOut = [fOut, 'shaded_relief']
	endif

	if keyword_set(ALL) then begin
		bptr = [0,1,2,3,4,5,6,7,8,9]
		fOut = ['slope','aspect','shaded_relief','profile_convexity','plan_convexity','long_convexity','cross_convexity','min_curvature','max_curvature','rms']
	endif

	;salgo si no he establecido la generacion de algún subproducto
	if (n_elements(bptr) eq 1 and bptr[0] eq -1) then return

	if not keyword_set(ALL) then begin
		bptr = bptr[1:*]
		fOut = fOut[1:*]
	endif

	;Seteo de algunos parametros de entrada que dependen de lo que se quiera generar
	kernelSize = long(i_kernel)

	shrelief_azimuth = float(i_shaded_relief_azimuth)
	shrelief_elevation = float(i_shaded_relief_elevation)

	;inicio del procesamiento
	envi_batch_init

	;abro el archivo del MDE
	envi_open_file, fInputDEM, r_fid=fid
	if (fid eq -1) then errorHandler, 'File ' + fInputDEM + ' not found', 0

	;consulto las dimensiones del archivo
	envi_file_query, fid, dims=dims, nl=nl
	
	;para manejar distintas bandas (suponemos que tenemos una sola)
	pos = [0]
	
	;para administrar la informacion geografica
	proj = envi_get_projection(fid=fid, pixel_size=pixel_size)

	;llamamos a la rutina de envi que calcula los subproductos
	for i=0l, n_elements(bptr)-1 do begin
		if (bptr[i] ne 2) then begin
			envi_doit, 'topo_doit', fid=fid, pos=pos, dims=dims, bptr=bptr[i], out_name=nSite + '_' + fOut[i], pixel_size=pixel_size, r_fid=r_fid, kernel_size=kernelSize
		endif else begin
			envi_doit, 'topo_doit', fid=fid, pos=pos, dims=dims, bptr=bptr[i], out_name=nSite + '_' + fOut[i], pixel_size=pixel_size, r_fid=r_fid, kernel_size=kernelSize, azimuth=shrelief_azimuth, elevation=shrelief_elevation
		endelse
	endfor
	
; stop	
	;para salir de envi
	;envi_batch_end

return
end
