;**********************************************************************
;WRITEROI
;**********************************************************************

pro writeROI, x, y, NODOS=NODOS, ARCOS=ARCOS, POLYGON=POLYGON, fOutput, ns, nl

	envi_batch_init

	roiID = envi_create_roi(color=255, name=fOutput, ns=ns, nl=nl)

	if keyword_set(ARCOS) then begin
		if keyword_set(POLYGON) then begin
			xp = [reform(x[arcos[0,*]]),x[arcos[0,0]]] & $
			yp = [reform(y[arcos[0,*]]),y[arcos[0,0]]] & $
			envi_define_roi, roiID, /polygon, xpts=xp+0.5, ypts=yp+0.5 & $
		endif else begin
			for j=0l,n_elements(arcos[0,*])-1 do begin & $
				z=arcos[*,j] & $
				envi_define_roi, roiID, /polyline, xpts=x[z]+0.5, ypts=y[z]+0.5 & $
			endfor
		endelse
	endif

	if keyword_set(NODOS) then begin
		for j=0l,n_elements(nodos)-1 do begin & $
			envi_define_roi, roiID, /point, xpts=x[nodos[j]], ypts=y[nodos[j]] & $
		endfor
	endif

	envi_save_rois, fOutput + '.roi', roiID
	envi_delete_rois, roiID
return
end

;**********************************************************************
;READDATA
;**********************************************************************

pro readData, file, data, DIMS=DIMS, NL=NL, MAPINFO=MAPINFO

	;valor que devuelve si no encuentra el archivo
	data = -1

	envi_batch_init

	envi_open_file, file, r_fid=fid
	if (fid eq -1) then begin
		errorHandler, 'File ' + file + ' not found', 0
	endif

	;consulto las dimensiones del archivo
	envi_file_query, fid, dims=dims, nl=nl
		
	;para manejar distintas bandas (suponemos que tenemos una sola)
	pos = [0]
		
	;para administrar la informacion geografica
	;proj = envi_get_projection(fid=fid, pixel_size=pixel_size)

	;para administra informacion de mapeo
	mapInfo = envi_get_map_info(fid=fid)
	
	;leo los datos del archivo
	data = envi_get_data(dims=dims, fid=fid, pos=pos)

	;retorno datos auxiliares
	if keyword_set(DIMS) then DIMS=dims
	if keyword_set(NL) then NL=nl
	if keyword_set(MAPINFO) then MAPINFO=mapInfo
return
end


